import { Box } from '../../classes/display/box.js';
import { Scene } from '../../classes/display/scene.js';
import { WeightedDirectedGraph } from '../../classes/supporting/wdg.js';
import { mochaRun } from '../../util/helpers.js';

const rootElement = document.getElementById('scene');
const rootBox = new Box('rootBox', rootElement).flex();
window.scene = new Scene('WDG test', rootBox);

describe('Weighted Directed Graph', function tests() {
  this.timeout(0);

  let graph;

  before(() => {
    graph = (window.graph = new WeightedDirectedGraph('test1', window.scene)).withFlowchart();

    graph.addVertex('v1', {});
    graph.addVertex('v1', {});
    graph.addVertex('v1', {});
    graph.addVertex('v1', {});
    graph.addVertex('v1', {});

    graph.addEdge('e1', 0, 1, 1);
    graph.addEdge('e1', 2, 1, 0.5);
    graph.addEdge('e1', 3, 1, 0.25);
    graph.addEdge('e1', 1, 4, 0.125);
  });

  it('can query for all e1 edges', () => {
    const edges = graph.getEdges('e1');
    edges.should.have.length(4);
  });

  it('can query for all e1 edges from a particular vertex', () => {
    const edges = graph.getEdges('e1', 2);
    edges.map(({ from, to, weight }) => [from.id, to.id, weight]).should.have.deep.members([[
      '2', '1', 0.5,
    ]]);
  });

  it('can query for all e1 edges to a particular vertex', () => {
    const edges = graph.getEdges('e1', null, 1);
    edges.map(({ from, to, weight }) => [from.id, to.id, weight]).should.have.deep.members([
      ['0', '1', 1],
      ['2', '1', 0.5],
      ['3', '1', 0.25],
    ]);
  });

  it('can export to JSON', () => {
    const result = graph.toJSON();
    console.log('export to JSON, result', result);
    result.should.eql({
      vertices: [
        {
          id: '0', type: 'v1', label: '0', properties: {},
        },
        {
          id: '1', type: 'v1', label: '1', properties: {},
        },
        {
          id: '2', type: 'v1', label: '2', properties: {},
        },
        {
          id: '3', type: 'v1', label: '3', properties: {},
        },
        {
          id: '4', type: 'v1', label: '4', properties: {},
        },
      ],
      edges: [
        {
          from: '0',
          to: '1',
          type: 'e1',
          weight: 1,
        },
        {
          from: '2',
          to: '1',
          type: 'e1',
          weight: 0.5,
        },
        {
          from: '3',
          to: '1',
          type: 'e1',
          weight: 0.25,
        },
        {
          from: '1',
          to: '4',
          type: 'e1',
          weight: 0.125,
        },
      ],
      history: [],
    });
  });
});

describe('Editable', () => {
  let graph;

  it('should be editable', () => {
    graph = (window.graph2 = new WeightedDirectedGraph('test2', window.scene, { editable: true })).withFlowchart();

    graph.addVertex('v1', {});
    graph.addVertex('v2', {});
    graph.addVertex('v3', {});

    graph.addEdge('e1', 2, 1, 1);
    graph.addEdge('e2', 1, 0, 0.5);
    graph.addEdge('e3', 2, 0, 0.25);
  });
});

mochaRun();
